<?php

/**
 * 2007-2022 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2022 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */


use PrestaShop\PrestaShop\Core\Domain\Order\Exception\OrderException;
use \GuzzleHttp\Exception\ClientException;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

if (!defined('_PS_VERSION_')) {
    exit;
}

include_once(_PS_MODULE_DIR_ . 'correoargentino/src/loader.php');

class Correoargentino extends CarrierModule
{
    protected $config_form = false;
    const RATE_HOME_DELIVERY = 'Envío a domicilio';
    const ENABLE_EXTERNAL_RATES = false;

    public $id_carrier;


    public function __construct()
    {
        $this->name = 'correoargentino';
        $this->tab = 'shipping_logistics';
        $this->version = '1.2.8';
        $this->author = 'TIARG S.A.';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Correo Argentino');
        $this->description = $this->l('Shipping method module of Correo Argentino');
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);


    }


    /**
     * @return bool
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function install()
    {

        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        include(dirname(__FILE__) . "/sql/install.php");

        Configuration::updateValue('CORREOARGENTINO_SANDBOX_MODE', 1);
        Configuration::updateValue('CORREOARGENTINO_STATE_ORDER', 2);

        $this->registerCarriers();

        return parent::install() &&
            $this->installTab() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('displayCarrierExtraContent') &&
            $this->registerHook('actionOrderStatusUpdate') &&
            $this->registerHook('actionValidateStepComplete') &&
            $this->registerHook('actionGetAdminOrderButtons') &&
            $this->registerHook('actionValidateOrder') &&
            $this->registerHook('actionPresentOrder');
    }


    /**
     * @return bool
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function uninstall()
    {
        include(dirname(__FILE__) . "/sql/uninstall.php");
        Configuration::deleteByName('CORREOARGENTINO_SANDBOX_MODE');
        Configuration::deleteByName('CORREOARGENTINO_AGREEMENT');
        Configuration::deleteByName('CORREOARGENTINO_API_KEY');
        Configuration::deleteByName('CORREOARGENTINO_CITYNAME');
        Configuration::deleteByName('CORREOARGENTINO_DEPARTAMENT');
        Configuration::deleteByName('CORREOARGENTINO_FLOOR');
        Configuration::deleteByName('CORREOARGENTINO_STATE');
        Configuration::deleteByName('CORREOARGENTINO_STREET_NAME');
        Configuration::deleteByName('CORREOARGENTINO_STREET_NUMBER');
        Configuration::deleteByName('CORREOARGENTINO_ZIP_CODE');
        Configuration::deleteByName('CORREOARGENTINO_AREA_CODE_CELL_PHONE');
        Configuration::deleteByName('CORREOARGENTINO_CELL_PHONE_NUMBER');
        Configuration::deleteByName('CORREOARGENTINO_AREA_CODE_PHONE');
        Configuration::deleteByName('CORREOARGENTINO_PHONE_NUMBER');
        Configuration::deleteByName('CORREOARGENTINO_BUSINESS_NAME');
        Configuration::deleteByName('CORREOARGENTINO_EMAIL');
        Configuration::deleteByName('CORREOARGENTINO_OBSERVATION');
        Configuration::deleteByName('CORREOARGENTINO_STATE_ORDER');
        Configuration::deleteByName('CORREOARGENTINO_CARRIER_HOME_ID');
        Configuration::deleteByName('CORREOARGENTINO_CARRIER_BRANCH_ID');
        Configuration::deleteByName('CORREOARGENTINO_API_KEY');
        Configuration::deleteByName('CORREOARGENTINO_API_KEY_SUCCESS');
        Configuration::deleteByName('CORREOARGENTINO_STATE_ORDER');

        $this->unregisterCarriers();

        return parent::uninstall() && $this->uninstallTab();
    }


    /**
     * @return bool
     */
    private function installTab(): bool
    {
        $tab = new Tab();
        $tab->active = 1;
        $tab->class_name = 'AdminCorreoArgentino';
        $tab->position = 3;
        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = 'Correo Argentino';
        }
        $tab->id_parent = (int)Tab::getIdFromClassName('AdminParentOrders');
        $tab->module = $this->name;
        $tab->save();
        return true;
    }

    /**
     * @return bool
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    private function uninstallTab(): bool
    {
        $tabId = (int)Tab::getIdFromClassName('AdminCorreoArgentino');

        if (!$tabId) {
            return true;
        }

        $tab = new Tab($tabId);

        return $tab->delete();
    }

    public function disable($force_all = false)
    {
        $carriers = Carrier::getCarriers($this->context->language->id, true, false, false, null, Carrier::CARRIERS_MODULE);
        foreach ($carriers as $carrier) {
            if ($carrier['external_module_name'] == 'correoargentino') {
                $carrierObj = new Carrier($carrier['id_carrier']);
                $carrierObj->active = '0';
                $carrierObj->save();
            }
        }

        return parent::disable($force_all);
    }

    public function enable($force_all = false)
    {
        $carriers = Carrier::getCarriers($this->context->language->id, false, false, false, null, Carrier::CARRIERS_MODULE);
        foreach ($carriers as $carrier) {
            if ($carrier['external_module_name'] == 'correoargentino') {
                $carrierObj = new Carrier($carrier['id_carrier']);
                $carrierObj->active = '1';
                $carrierObj->save();
            }
        }

        return parent::enable($force_all);
    }


    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        $this->getConfigFormValues();

        $submitted = true;
        $errors = [];

        if ((Tools::isSubmit('submitCorreoargentinoModule') == true && Tools::isSubmit('submitCorreoargentinoModuleLogin')) == true) {
            try {
                $mode = Tools::getValue('CORREOARGENTINO_SANDBOX_MODE');
                Configuration::updateValue("CORREOARGENTINO_SANDBOX_MODE", $mode);
                if (empty(Tools::getValue("CORREOARGENTINO_AGREEMENT")) || empty(Tools::getValue("CORREOARGENTINO_API_KEY"))) {
                    throw new Exception($this->l('Agreement and API Key are required'));
                }
                $response = $this->postProcessLogin($mode);
                if (!$response) {
                    throw new Exception($this->l('Invalid credentials'));
                }
                $success = true;
            } catch (Exception $e) {
                $errors[] = $e->getMessage();
                Configuration::updateValue("CORREOARGENTINO_SANDBOX_MODE", null);
                Configuration::updateValue("CORREOARGENTINO_AGREEMENT", null);
                Configuration::updateValue("CORREOARGENTINO_API_KEY", null);
            }
        }

        if ((Tools::isSubmit('submitCorreoargentinoModule') == true && Tools::isSubmit('submitCorreoargentinoModuleStatus')) == true) {
            try {
                $this->postProcessStatus();
                $success = true;
            } catch (Exception $e) {
                $success = false;
            }
        }

        if ((Tools::isSubmit('submitCorreoargentinoModule') == true && Tools::isSubmit('submitCorreoargentinoModuleAddress')) == true) {


            try {
                $success = true;
                $zipCodeRegex = "/^([A-HJ-TP-Z]{1}\d{4}[A-Z]{3}|[a-z]{1}\d{4}[a-hj-tp-z]{3}|[0-9][0-9][0-9][0-9])$/";
                $zipCode = Tools::getValue('CORREOARGENTINO_ZIP_CODE');

                $streetNumberRegex = "/^[0-9]{1,4}$/";
                $streetNumber = Tools::getValue('CORREOARGENTINO_STREET_NUMBER');

                $streetNameRegex = "/^[0-9a-zA-ZÀ-ÿ\s]{1,32}$/";
                $streetName = Tools::getValue('CORREOARGENTINO_STREET_NAME');

                $cellphoneCodeRegex = "/^[0-9]{2}$/";
                $cellphoneCode = Tools::getValue('CORREOARGENTINO_AREA_CODE_CELL_PHONE');

                $cellphoneNumberRegex = "/^[0-9]{8,10}$/";
                $cellphoneNumber = Tools::getValue('CORREOARGENTINO_CELL_PHONE_NUMBER');

                $telephoneCodeRegex = "/^[0-9]{2}$/";
                $telephoneCode = Tools::getValue('CORREOARGENTINO_AREA_CODE_PHONE');

                $telephoneNumberRegex = "/^[0-9]{8,10}$/";
                $telephoneNumber = Tools::getValue('CORREOARGENTINO_PHONE_NUMBER');

                $citynameRegex = "/^[a-zA-ZÀ-ÿ\s]{1,32}$/";
                $cityname = Tools::getValue('CORREOARGENTINO_CITYNAME');

                $email = Tools::getValue('CORREOARGENTINO_EMAIL');

                $required = array(
                    'CORREOARGENTINO_BUSINESS_NAME',
                    'CORREOARGENTINO_CITYNAME',
                    'CORREOARGENTINO_ZIP_CODE',
                    'CORREOARGENTINO_STREET_NAME',
                    'CORREOARGENTINO_STREET_NUMBER',
                );

                $messages = array(
                    'CORREOARGENTINO_BUSINESS_NAME' => $this->l('The Business Name field is required.'),
                    'CORREOARGENTINO_CITYNAME' => $this->l('The City field is required.'),
                    'CORREOARGENTINO_ZIP_CODE' => $this->l('The Postal Code field is required.'),
                    'CORREOARGENTINO_STREET_NAME' => $this->l('The Street Name is required.'),
                    'CORREOARGENTINO_STREET_NUMBER' => $this->l('The Street Number is required.')
                );

                $matches = array_map(function ($key) {
                    $input = Tools::getValue("$key");
                    return empty($input);
                }, $required);


                $filtered = array_filter($matches);

                $keys = array_keys($filtered);

                if (in_array(true, $matches)) {
                    foreach ($keys as $key => $value) {
                        $errors[] = $messages[$required[$value]];
                    }
                }


                if (!preg_match($zipCodeRegex, $zipCode)) {
                    $errors[] = $this->l('The Postal Code field is invalid');
                    $success = false;
                }

                if (!preg_match($streetNumberRegex, $streetNumber)) {
                    $errors[] = $this->l('The Street Number field must be numeric');
                    $success = false;
                }

                if (!preg_match($streetNameRegex, $streetName)) {
                    $errors[] = $this->l('The Street Name field does not accept special characters');
                    $success = false;
                }

                if (!empty($cellphoneCode) && !preg_match($cellphoneCodeRegex, $cellphoneCode)) {
                    $errors[] = $this->l('The Cellphone Code Area field must be numeric');
                    $success = false;
                }

                if (!empty($cellphoneNumber) && !preg_match($cellphoneNumberRegex, $cellphoneNumber)) {
                    $errors[] = $this->l('The Cellphone Number field must be numeric');
                    $success = false;
                }

                if (!empty($telephoneCode) && !preg_match($telephoneCodeRegex, $telephoneCode)) {
                    $errors[] = $this->l('The Phone Code Area field must be numeric');
                    $success = false;
                }

                if (!empty($telephoneNumber) && !preg_match($telephoneNumberRegex, $telephoneNumber)) {
                    $errors[] = $this->l('The Phone Number field must be numeric');
                    $success = false;
                }

                if (!empty($cityname) && !preg_match($citynameRegex, $cityname)) {
                    $errors[] = $this->l('The City field does not accept special characters');
                    $success = false;
                }

                if (!empty($email) && !filter_var($email, FILTER_VALIDATE_EMAIL)) {
                    $errors[] = $this->l('The Email field is not valid');
                    $success = false;
                }

                if (count($errors) < 1) {
                    $this->postProcessAddress();
                }

            } catch (Exception $e) {
                $errors[] = $e->getMessage();
                $success = false;
            }
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl', compact('success', 'submitted', 'errors'));


        return $output . $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitCorreoargentinoModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&step=1';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            /* Add values for your inputs */
            'fields_value' => $this->getConfigFormValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        if (Configuration::get('CORREOARGENTINO_API_KEY_SUCCESS') && !empty(Configuration::get('CORREOARGENTINO_API_KEY'))) {
            $forms = [$this->getConfigForm(), $this->getConfigFormAddress(), $this->getStatusesForm()];
        } else {
            $forms = [$this->getConfigForm()];
        }
        return $helper->generateForm($forms);
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'collapsible' => true,
                'legend' => array(
                    'title' => $this->l('API Credentials'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'col' => 4,
                        'type' => 'text',
                        'name' => 'CORREOARGENTINO_AGREEMENT',
                        'label' => $this->l('Agreement'),
                    ),
                    array(
                        'col' => 6,
                        'type' => 'textarea',
                        'name' => 'CORREOARGENTINO_API_KEY',
                        'desc' => $this->l('Don\'t have an account yet?') . " " . "<a href='https://www.correoargentino.com.ar/'>" . $this->l('Create a new account') . "</a>",
                        'label' => $this->l('API Key'),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Sandbox'),
                        'name' => 'CORREOARGENTINO_SANDBOX_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Enable this if you want to work in test mode'),
                        'values' => array(
                            array(
                                'id' => 'label2_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'label2_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        )
                    )
                ),
                'submit' => array(
                    'name' => 'submitCorreoargentinoModuleLogin',
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }


    /**
     * Create the structure of your form.
     */
    protected function getConfigFormAddress()
    {
        $country = Configuration::get('PS_COUNTRY_DEFAULT');
        $states = array_map(function ($c) {
            return ['id' => $c['iso_code'], 'name' => $c['name']];
        }, State::getStatesByIdCountry($country));
        return [
            'form' => [
                'legend' => [
                    'title' => $this->l('Shop Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'col' => 6,
                        'type' => 'text',
                        'maxlength' => 120,
                        'name' => 'CORREOARGENTINO_BUSINESS_NAME',
                        'label' => $this->l('Business name'),
                        'required' => true,
                        'desc' => $this->l('You can use a personal or business name.')
                    ],
                    [
                        'col' => 4,
                        'type' => 'text',
                        'constraint' => "isEmail",
                        'maxlength' => 120,
                        'hint' => $this->l('Enter the following format:') . ' correoargentino@correo.com',
                        'name' => 'CORREOARGENTINO_EMAIL',
                        'label' => $this->l('E-mail'),
                        'required' => false
                    ],
                    [
                        'col' => 4,
                        'type' => 'select',
                        'name' => 'CORREOARGENTINO_STATE',
                        'label' => $this->l('State'),
                        'required' => false,
                        'options' => [
                            'query' => $states,
                            'id' => 'id',
                            'name' => 'name'
                        ]
                    ],
                    [
                        'col' => 6,
                        'type' => 'text',
                        'maxlength' => 64,
                        'name' => 'CORREOARGENTINO_CITYNAME',
                        'label' => $this->l('City'),
                        'required' => true
                    ],
                    [
                        'col' => 2,
                        'type' => 'text',
                        'maxlength' => 2,
                        'name' => 'CORREOARGENTINO_DEPARTAMENT',
                        'label' => $this->l('Department'),
                        'required' => false
                    ],
                    [
                        'col' => 2,
                        'type' => 'text',
                        'maxlength' => 2,
                        'name' => 'CORREOARGENTINO_FLOOR',
                        'label' => $this->l('Floor'),
                        'required' => false
                    ],
                    [

                        'col' => 6,
                        'type' => 'text',
                        'maxlength' => 64,
                        'name' => 'CORREOARGENTINO_STREET_NAME',
                        'label' => $this->l('Street'),
                        'required' => true
                    ],
                    [
                        'col' => 2,
                        'type' => 'text',
                        'maxlength' => 6,
                        'name' => 'CORREOARGENTINO_STREET_NUMBER',
                        'label' => $this->l('Street number'),
                        'required' => true
                    ],
                    [
                        'col' => 4,
                        'type' => 'text',
                        'maxlength' => 8,
                        'name' => 'CORREOARGENTINO_ZIP_CODE',
                        'label' => $this->l('Postal code'),
                        'required' => true,
                        'desc' => $this->l('Please enter your postal code. You can find it in ') . "<a href='https://www.correoargentino.com.ar/formularios/cpa'>" . $this->l('here') . "</a>",
                        'hint' => $this->l('You can enter only the numeric part')
                    ],
                    [
                        'col' => 2,
                        'type' => 'text',
                        'maxlength' => 2,
                        'name' => 'CORREOARGENTINO_AREA_CODE_CELL_PHONE',
                        'label' => $this->l('Cellphone area code'),
                        'required' => false
                    ],
                    [
                        'col' => 6,
                        'type' => 'text',
                        'maxlength' => 16,
                        'name' => 'CORREOARGENTINO_CELL_PHONE_NUMBER',
                        'label' => $this->l('Cellphone'),
                        'required' => false
                    ],
                    [
                        'col' => 2,
                        'type' => 'text',
                        'maxlength' => 2,
                        'name' => 'CORREOARGENTINO_AREA_CODE_PHONE',
                        'label' => $this->l('Telephone area code'),
                        'required' => false
                    ],
                    [
                        'col' => 6,
                        'type' => 'text',
                        'maxlength' => 16,
                        'name' => 'CORREOARGENTINO_PHONE_NUMBER',
                        'label' => $this->l('Telephone'),
                        'required' => false
                    ],
                    [
                        'col' => 6,
                        'type' => 'textarea',
                        'name' => 'CORREOARGENTINO_OBSERVATION',
                        'maxlength' => 500,
                        'label' => $this->l('Observation'),
                        'required' => false,
                        'desc' => $this->l('Enter the details to help us to find your address')
                    ],
                ],
                'submit' => [
                    'name' => 'submitCorreoargentinoModuleAddress',
                    'title' => $this->l('Save'),
                ]
            ]
        ];
    }


    /**
     * @return array[]
     */
    protected function getStatusesForm(): array
    {
        $states = new PrestaShop\PrestaShop\Adapter\OrderState\OrderStateDataProvider();
        $statesList = $states->getOrderStates(Configuration::get('PS_LANG_DEFAULT'));

        $optionsQuery = array_map(
            function ($v) {
                return [
                    'id' => $v['id_order_state'],
                    'name' => $v['name']
                ];
            },
            $statesList
        );

        $optionsQuery[] = [
            'id' => '',
            'name' => 'Sin asignar'
        ];

        return [
            'form' => [
                'name' => 'anotherXForm',
                'legend' => [
                    'title' => $this->l('Shipment Trigger'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'col' => 6,
                        'type' => 'select',
                        'name' => 'CORREOARGENTINO_STATE_ORDER',
                        'label' => $this->l('Order state'),
                        'desc' => $this->l('This will be the state which will create a new order into Correo Argentino API'),
                        'required' => true,
                        'options' => [
                            'query' => $optionsQuery,
                            'id' => 'id',
                            'name' => 'name'
                        ]
                    ]
                ],
                'submit' => [
                    'name' => 'submitCorreoargentinoModuleStatus',
                    'title' => $this->l('Save'),
                ]
            ]
        ];
    }


    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'CORREOARGENTINO_SANDBOX_MODE' => Configuration::get('CORREOARGENTINO_SANDBOX_MODE'),
            'CORREOARGENTINO_AGREEMENT' => Configuration::get('CORREOARGENTINO_AGREEMENT'),
            'CORREOARGENTINO_API_KEY' => Configuration::get('CORREOARGENTINO_API_KEY'),
            'CORREOARGENTINO_CITYNAME' => Configuration::get('CORREOARGENTINO_CITYNAME'),
            'CORREOARGENTINO_DEPARTAMENT' => Configuration::get('CORREOARGENTINO_DEPARTAMENT'),
            'CORREOARGENTINO_FLOOR' => Configuration::get('CORREOARGENTINO_FLOOR'),
            'CORREOARGENTINO_STATE' => Configuration::get('CORREOARGENTINO_STATE'),
            'CORREOARGENTINO_STREET_NAME' => Configuration::get('CORREOARGENTINO_STREET_NAME'),
            'CORREOARGENTINO_STREET_NUMBER' => Configuration::get('CORREOARGENTINO_STREET_NUMBER'),
            'CORREOARGENTINO_ZIP_CODE' => Configuration::get('CORREOARGENTINO_ZIP_CODE'),
            'CORREOARGENTINO_AREA_CODE_CELL_PHONE' => Configuration::get('CORREOARGENTINO_AREA_CODE_CELL_PHONE'),
            'CORREOARGENTINO_CELL_PHONE_NUMBER' => Configuration::get('CORREOARGENTINO_CELL_PHONE_NUMBER'),
            'CORREOARGENTINO_AREA_CODE_PHONE' => Configuration::get('CORREOARGENTINO_AREA_CODE_PHONE'),
            'CORREOARGENTINO_PHONE_NUMBER' => Configuration::get('CORREOARGENTINO_PHONE_NUMBER'),
            'CORREOARGENTINO_BUSINESS_NAME' => Configuration::get('CORREOARGENTINO_BUSINESS_NAME'),
            'CORREOARGENTINO_EMAIL' => Configuration::get('CORREOARGENTINO_EMAIL'),
            'CORREOARGENTINO_OBSERVATION' => Configuration::get('CORREOARGENTINO_OBSERVATION'),
            'CORREOARGENTINO_STATE_ORDER' => Configuration::get('CORREOARGENTINO_STATE_ORDER')
        );
    }

    /**
     * Save form data.
     */
    protected function postProcessLogin($mode)
    {
        try {
            $agreement = str_replace(' ', '', Tools::getValue('CORREOARGENTINO_AGREEMENT'));
            $apiKey = str_replace(' ', '', Tools::getValue('CORREOARGENTINO_API_KEY'));
            $service = new CorreoArgentinoService();
            $response = $service->login($agreement, $apiKey, $mode);
            if ($response) {
                Configuration::updateValue("CORREOARGENTINO_AGREEMENT", $agreement);
                Configuration::updateValue("CORREOARGENTINO_API_KEY", $apiKey);
            }
            return $response;
        } catch (Exception | TransportExceptionInterface $e) {
            Configuration::updateValue("CORREOARGENTINO_AGREEMENT", null);
            Configuration::updateValue("CORREOARGENTINO_API_KEY", null);
            Configuration::updateValue("CORREOARGENTINO_API_KEY_SUCCESS", false);
            PrestaShopLogger::addLog($e->getMessage());
            return false;
        }
    }

    /**
     *
     */
    protected function postProcessAddress()
    {
        Configuration::updateValue('CORREOARGENTINO_CITYNAME', Tools::getValue('CORREOARGENTINO_CITYNAME'));
        Configuration::updateValue('CORREOARGENTINO_DEPARTAMENT', Tools::getValue('CORREOARGENTINO_DEPARTAMENT'));
        Configuration::updateValue('CORREOARGENTINO_FLOOR', Tools::getValue('CORREOARGENTINO_FLOOR'));
        Configuration::updateValue('CORREOARGENTINO_STATE', Tools::getValue('CORREOARGENTINO_STATE'));
        Configuration::updateValue('CORREOARGENTINO_STREET_NAME', Tools::getValue('CORREOARGENTINO_STREET_NAME'));
        Configuration::updateValue('CORREOARGENTINO_STREET_NUMBER', Tools::getValue('CORREOARGENTINO_STREET_NUMBER'));
        Configuration::updateValue('CORREOARGENTINO_ZIP_CODE', Tools::getValue('CORREOARGENTINO_ZIP_CODE'));
        Configuration::updateValue('CORREOARGENTINO_AREA_CODE_CELL_PHONE', Tools::getValue('CORREOARGENTINO_AREA_CODE_CELL_PHONE'));
        Configuration::updateValue('CORREOARGENTINO_CELL_PHONE_NUMBER', Tools::getValue('CORREOARGENTINO_CELL_PHONE_NUMBER'));
        Configuration::updateValue('CORREOARGENTINO_AREA_CODE_PHONE', Tools::getValue('CORREOARGENTINO_AREA_CODE_PHONE'));
        Configuration::updateValue('CORREOARGENTINO_PHONE_NUMBER', Tools::getValue('CORREOARGENTINO_PHONE_NUMBER'));
        Configuration::updateValue('CORREOARGENTINO_BUSINESS_NAME', Tools::getValue('CORREOARGENTINO_BUSINESS_NAME'));
        Configuration::updateValue('CORREOARGENTINO_EMAIL', Tools::getValue('CORREOARGENTINO_EMAIL'));
        Configuration::updateValue('CORREOARGENTINO_OBSERVATION', Tools::getValue('CORREOARGENTINO_OBSERVATION'));
    }

    /**
     *
     */
    protected function postProcessStatus()
    {
        Configuration::updateValue('CORREOARGENTINO_STATE_ORDER', Tools::getValue('CORREOARGENTINO_STATE_ORDER'));
    }

    /**
     * @param $params
     * @param $shipping_cost
     * @return float
     * @throws PrestaShopDatabaseException
     */
    public function getOrderShippingCost($params, $shipping_cost)
    {
        return $this->getOrderShippingCostExternal($params);
    }


    /**
     * @param $params
     * @return float
     * @throws PrestaShopDatabaseException
     */
    public function getOrderShippingCostExternal($params)
    {
        /*
        * @todo: we have to consume the /rates endpoint in order to get the actual values
        * It's necessary to get a unique key for each record in order to associate with its carrier method
        */
        $value = self::ENABLE_EXTERNAL_RATES ? CorreoArgentinoRatesModel::getRateByCarrierId($this->id_carrier) : 0;
        return empty($value) ? false : round((float)$value, 2);
    }


    /**
     * @param $type
     * @return Carrier|false
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    protected function addCarrier($type)
    {
        $lang_id = Configuration::get('PS_LANG_DEFAULT');
        $carrier = new Carrier();
        $carrier->name = $this->l('Correo Argentino');
        $carrier->is_module = true;
        $carrier->active = 1;
        $carrier->need_range = 1;
        $carrier->is_free = !self::ENABLE_EXTERNAL_RATES; // If is set this prevents to add default values;
        $carrier->shipping_external = self::ENABLE_EXTERNAL_RATES;
        $carrier->range_behavior = 0;
        $carrier->external_module_name = $this->name;
        $carrier->shipping_method = 2;
        $carrier->url = 'https://www.correoargentino.com.ar/formularios/e-commerce?id=@';

        $delay = $type == self::RATE_HOME_DELIVERY ? $this->l('Home delivery') : $this->l('Agency');
        $carrier->delay[$lang_id] = $delay;

        if ($carrier->add() == true) {
            @copy(dirname(__FILE__) . '/views/img/ca_logo_240x240.jpeg', _PS_SHIP_IMG_DIR_ . '/' . (int)$carrier->id . '.jpg');
            return $carrier;
        }

        return false;
    }


    /**
     * @param $carrier
     */
    protected function addGroups($carrier)
    {
        $groups_ids = array();
        $groups = Group::getGroups(Context::getContext()->language->id);
        foreach ($groups as $group)
            $groups_ids[] = $group['id_group'];

        $carrier->setGroups($groups_ids);
    }

    /**
     * @param $carrier
     */
    protected function addRanges($carrier)
    {
        $range_price = new RangePrice();
        $range_price->id_carrier = $carrier->id;
        $range_price->delimiter1 = '0';
        $range_price->delimiter2 = '10000';
        $range_price->add();

        $range_weight = new RangeWeight();
        $range_weight->id_carrier = $carrier->id;
        $range_weight->delimiter1 = '0';
        $range_weight->delimiter2 = '10000';
        $range_weight->add();
    }

    /**
     * @param $carrier
     */
    protected function addZones($carrier)
    {
        // Set South America as delivery region
        $country_id = Country::getByIso('AR');
        $zone_id = Country::getIdZone($country_id);
        $carrier->addZone($zone_id);
    }


    /**
     * @param $carrier
     * @throws PrestaShopException
     */
    protected function addRates($carrier, $price, $isBranch)
    {
        CorreoArgentinoRatesModel::create($carrier, $price, $isBranch);
    }


    /**
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    protected function registerCarriers()
    {

        $rates = [
            "rates" => [
                [
                    "serviceName" => "Correo Argentino 01",
                    "description" => "Envío a domicilio",
                    "is_rate" => false,
                    "serviceCode" => "EP",
                    "currency" => "ARS",
                    "totalPrice" => "500"
                ],
                [
                    "serviceName" => "Correo Argentino 02",
                    "description" => "Envio a sucursal de CA",
                    "is_rate" => true,
                    "serviceCode" => "EP",
                    "currency" => "ARS",
                    "totalPrice" => "400"
                ],
                [
                    "serviceName" => "Correo Argentino 03",
                    "description" => "Envío a domicilio",
                    "is_rate" => false,
                    "serviceCode" => "EP",
                    "currency" => "ARS",
                    "totalPrice" => "300"
                ],
                [
                    "serviceName" => "Correo Argentino 04",
                    "description" => "Envio a sucursal de CA",
                    "is_rate" => true,
                    "serviceCode" => "EP",
                    "currency" => "ARS",
                    "totalPrice" => "200"
                ]
            ]
        ];


        foreach ($rates['rates'] as $i => $rate) {
            if ($i > 1 && !self::ENABLE_EXTERNAL_RATES) {
                break;
            }
            $newCarrier = $this->addCarrier($rate['description'], $rate['is_rate']);
            $this->addZones($newCarrier);
            $this->addGroups($newCarrier);
            $this->addRanges($newCarrier);
            $this->addRates($newCarrier, $rate['totalPrice'], $rate['is_rate']);
        }
    }

    /**
     * @throws PrestaShopDatabaseException
     */
    protected function getRegisteredCarriersId()
    {
        $sql = new DbQuery();
        $sql->select('id_carrier');
        $sql->from('carrier', 'c');
        $sql->where('c.external_module_name = "correoargentino"');
        $sql->where('c.deleted = 0');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * @throws PrestaShopDatabaseException
     */
    protected function unregisterCarriers()
    {
        $carriers = $this->getRegisteredCarriersId();
        foreach ($carriers as $c) {
            $carrier = new Carrier($c['id_carrier']);
            $carrier->delete();
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }


    /**
     * @param $params
     * @throws PrestaShopException
     */
    public function hookActionOrderStatusUpdate($params)
    {
        try {

            $newOrderStatus = $params['newOrderStatus'];

            $id_order = $params['id_order'];

            if ((int)Configuration::get('CORREOARGENTINO_STATE_ORDER') !== $newOrderStatus->id) {
                return;
            }

            $order = new Order((int)$id_order);
            $carrier = new Carrier($order->id_carrier);

            /**
             * Solo podemos procesar ordenes del modulo
             */
            if ($carrier->external_module_name != "correoargentino") {
                return;
            }

            /**
             * Validacion para no registrar duplicado
             */
            if (!empty($order->getWsShippingNumber())) {
                if ($this->context->controller) {
                    $this->context->controller->confirmations[] = $this->l('successful registration in Correo Argentino');
                }
                return;
            }

            $service = new CorreoArgentinoService();

            $response = $service->registerOrder($order);

            if ($response instanceof Exception) {
                throw new OrderException($this->l($response->getResponse()->getContent()));
            } else {
                $this->context->controller->confirmations[] = $this->l('successful registration in Correo Argentino');
            }


        } catch (ClientException $e) {
            $exception = json_decode((string)$e->getResponse()->getBody(), true);
            $this->context->controller->errors[] = $exception["message"];
        } catch (PrestashopException $e) {
            $this->context->controller->errors[] = $e->getMessage();
        }

    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addJS($this->_path . '/views/js/select2.min.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');
        $this->context->controller->addCSS($this->_path . '/views/css/select2.min.css');
    }


    /**
     * @throws PrestaShopDatabaseException
     */
    public function hookDisplayCarrierExtraContent($params)
    {
        $carrier = $params['carrier'];
        $carrier_id = $carrier['id'];
        $carrier_ids = array_map(function ($item) {
            return (int)$item['id_carrier'];
        }, $this->getRegisteredCarriersId());

        if (!in_array($carrier_id, $carrier_ids) || $carrier['delay'] != $this->l('Agency')) {
            return;
        }
        $is_enabled = Module::isEnabled($this->name);
        $is_enabled_mobile = Module::isEnabledForMobileDevices($this->name);
        $this->smarty->assign(compact('carrier_id', 'is_enabled', 'is_enabled_mobile'));
        return $this->display(__FILE__, 'display-carrier-extra-content.tpl');
    }

    public function hookActionPresentOrder($params)
    {
        if (Module::isEnabled($this->name) == false) {
            return false;
        }
    }

    /**
     * @param $params
     * @return false
     * @throws PrestaShopException
     */
    public function hookActionValidateStepComplete($params)
    {
        $step_name = $params['step_name'];

        if (Module::isEnabled($this->name) == false) {
            return;
        }

        if ($step_name == 'delivery') {
            $request_params = $params['request_params'];
            $cart = $params['cart'];
            $id_carrier = $cart->id_carrier;
            $isBranch = CorreoArgentinoRatesModel::isBranch($id_carrier);
            $branch_id = $isBranch ? $request_params["correoargentino_branch_id_" . $id_carrier] : null;
            $branch = CorreoArgentinoBranchOrderModel::findOrCreate($cart->id);
            $branch->branch_code = $isBranch ? $branch_id : null;
            $branch->branch_select = $isBranch ? $id_carrier : null;
            $branch->id_order = null;
            $branch->tracking = null;
            $branch->total = CorreoArgentinoRatesModel::getRateByCarrierId($id_carrier);
            $branch->save();
        }
    }

    /**
     *
     * @throws PrestaShopException
     */
    public function hookActionValidateOrder($params)
    {
        $order = $params['order'];
        $carrier = new Carrier($order->id_carrier);

        if ($carrier->external_module_name != "correoargentino" || Module::isEnabled($this->name) == false) {
            return;
        }

        $isBranch = CorreoArgentinoRatesModel::isBranch($order->id_carrier);
        $branch = CorreoArgentinoBranchOrderModel::findOrCreate($order->id_cart);
        $branch->id_order = $order->id;
        $branch->shipping_type = $isBranch ? 'agency' : 'homeDelivery';
        $branch->save();
    }

    /**
     * Add buttons to main buttons bar
     */
    public function hookActionGetAdminOrderButtons(array $params)
    {
        $order = new Order($params['id_order']);

        if (!CorreoArgentinoRatesModel::isCaOrder($order->id_carrier) || !$order->getWsShippingNumber() || $this->checkOrderState($order, 'Canceled')) {
            return false;
        }

        $bar = $params['actions_bar_buttons_collection'];

        $cancelAnOrderUrl = Context::getContext()->link->getAdminLink('AdminCorreoArgentino', true, null, ['vieworder' => 'cancel', 'id_order' => $order->id]);
        $bar->add(
            new \PrestaShopBundle\Controller\Admin\Sell\Order\ActionsBarButton(
                'btn-action', ['href' => $cancelAnOrderUrl], '<i class="material-icons" aria-hidden="true">block</i> ' . $this->l('Cancel')
            )
        );

        $labelAnOrderUrl = Context::getContext()->link->getAdminLink('AdminCorreoArgentino', true, null, ['vieworder' => 'pdf', 'id_order' => $order->id]);
        $bar->add(
            new \PrestaShopBundle\Controller\Admin\Sell\Order\ActionsBarButton(
                'btn-action', ['href' => $labelAnOrderUrl], '<i class="material-icons" aria-hidden="true">picture_as_pdf</i>' . $this->l('Label')
            )
        );

    }

    public function checkOrderState($order, $state): bool
    {
        return in_array($state, $order->getCurrentOrderState()->name);
    }

}
